;*****************************************
; Monodirectional Serial Interface
; MIDI -> rs232 
; Qz 10Mhz clock
; Program: A.Piazza
; Date: 22/02/2003
; Release: 00
; Source: 232_pic.asm
; Note: pic side rs232
; porta 0=out serial data    
;       1=
;       2= 
;       3= 
;       4= 
; portb 0/7=byte parallel in
;*****************************************

          PROCESSOR       16f84a
          RADIX           hex
          INCLUDE         "P16F84a.INC"
          ERRORLEVEL      -302
          ERRORLEVEL      -305
          __CONFIG 3FFAH

;*****************************************
; 	 Port Assignments
;*****************************************

PCL	    equ 0x02
STATUS	    equ 0x03
PORTA	    equ	0x05
PORTB	    equ	0x06

;*****************************************
;         RS232 
;*****************************************

TX           equ  0       ;Tx data
RX           equ  1       ;Rx data

;*****************************************
; Clock frequency related constant
;       val .14 & 10mhz  = 38400 bit
;*****************************************

BIT_DELAY        equ  .14      ;Bit delay a 38400bps

;*****************************************
; MACRO - Delay subroutine with watch dog timer clearing
; Macro parameters:
;*****************************************

DELAY           MACRO   VALUE

                LOCAL   REDO
                movlw   VALUE
                movwf   TmpRegister
REDO            clrwdt                  ;Clear watch dog timer
                decfsz  TmpRegister,F
                goto    REDO

                ENDM
 
;***************************************** 
;              FILE REGISTER
;***************************************** 

                ORG     0x0C       ;Register used by msDelay subroutine and DELAY macro
msDelayCounter  res     0x02
TmpRegister     equ     0x0E       ;Register used by midi subroutines
ShiftReg        equ     0x0F       ;Shift register
BitCount        equ     0x10       ;Bit counter
DummyReg        equ     0x11
portb_tmp       equ     0x12

;*****************************************
;           Initialise Software
;*****************************************

                org    0x00 
                bsf    STATUS,RP0
                clrf   PORTA
                movlw  0x1A           ;PORTA B00011010
                movwf  TRISA 
 
                clrf   PORTB
                movlw  0xFF           ;PORTB B11111111 
                movwf  TRISB
                bcf    STATUS,RP0

;*****************************************
;	 Main from parallel PORTB
;*****************************************
  	
Main            movf    PORTB,w           ;see if same
                xorwf   portb_tmp,w       ;because already send 
                btfsc   STATUS,Z
                goto    Main              ;go up

                call    ParData           ;new byte midi
                call    TxChar            ;TX data
                goto    Main         

;*****************************************
; Delay subroutine
; W = Requested delay time in ms (clock = 10MHz)
;*****************************************

msDelay         movwf   msDelayCounter+1
                clrf    msDelayCounter+0 ; 1 ms (about) internal loop
                
msDelayLoop     decfsz  msDelayCounter+0,F
                nop
                goto    msDelayLoop
                nop
                decfsz  msDelayCounter+1,F
                goto    msDelayLoop
                return

;*****************************************
; Send a character on RS232
; Input: W = Character to send
;*****************************************

TxChar          movwf   ShiftReg
                movlw   8               ;Data lenght
                movwf   BitCount
                bcf     PORTA,TX        ;Send start bit
                nop
                nop
                nop
                nop
                nop
                nop
                nop
                DELAY   BIT_DELAY       ;Tx loop

TxLoop          btfss   ShiftReg,0
                goto    TxLo
                nop
                bsf     PORTA,TX
                goto    cTx

TxLo            bcf     PORTA,TX
                goto    cTx

cTx             rrf     ShiftReg,F
                nop
                DELAY   BIT_DELAY
                decfsz  BitCount,F
                goto    TxLoop
                nop
                nop
                nop
                nop
                bsf     PORTA,TX                ;Stop bit
                DELAY   BIT_DELAY
                DELAY   2
                nop
                bsf     PORTA,TX
                DELAY   BIT_DELAY
                DELAY   2
                return

;*****************************************
; Receive a character from RS232
; ShiftReg:  Received character
;*****************************************

RxChar          clrf    ShiftReg
                movlw   8                       ;Data lenght
                movwf   BitCount
                DELAY   BIT_DELAY+BIT_DELAY/2   ;Wait 1.5 bit
                
wDB             btfss   PORTA,RX ;Loop di lettura dei bit dati
                goto    RxBitL

RxBitH          bsf     STATUS,C
                nop
                goto    RxShift

RxBitL          bcf     STATUS,C
                goto    RxShift

RxShift         rrf     ShiftReg,F
                nop
                DELAY   BIT_DELAY
                decfsz  BitCount,F
                goto    wDB
                return

;*****************************************
;	 Data from parallel PORTB
;*****************************************

ParData         movf    PORTB,w           ;in w PB 
                movwf   portb_tmp
                return

                END

