;*****************************************
; Monodirectional Serial Interface
; MIDI -> rs232
; Qz 10Mhz clock
; Program: A.Piazza
; Date: 23/02/2003
; Release: 00
; Source: mid_pic.asm
; Note: no buffer data request
; porta 0= 
;       1=in  serial data                 
;       2=
;       3=
;       4=
; portb 0/7=byte parallel out
;*****************************************

          PROCESSOR       16f84a
          RADIX           hex
          INCLUDE         "P16F84a.INC"
          ERRORLEVEL      -302
          ERRORLEVEL      -305
          __CONFIG 3FFAH

;*****************************************
; 	 Port Assignments
;*****************************************

PCL	    equ 0x02
STATUS	    equ 0x03
PORTA	    equ	0x05
PORTB	    equ	0x06

;*****************************************
;         MIDI DATA PORT
;*****************************************

TX           equ  0       ;Tx data
RX           equ  1       ;Rx data

;*****************************************
; Clock frequency related constant
;       val .17 & 10mhx  = 31250 bit
;       val .14 & 10mhz  = 38400 bit
;*****************************************

BIT_DELAY    equ  .17      ;Bit delay a 31250bps

;*****************************************
; MACRO - Delay subroutine with watch dog timer clearing
; Macro parameters:
;*****************************************

DELAY           MACRO   VALUE

                LOCAL   REDO
                movlw   VALUE
                movwf   TmpRegister
REDO            clrwdt                  ;Clear watch dog timer
                decfsz  TmpRegister,F
                goto    REDO

                ENDM
 
;***************************************** 
;              FILE REGISTER
;***************************************** 

                ORG     0x0C    ;Register used by msDelay subroutine and DELAY macro

msDelayCounter  res     2
TmpRegister     res     1       ;Register used data subroutines
ShiftReg        res     1       ;Shift register
BitCount        res     1       ;Bit counter
DummyReg        res     1

;*****************************************
;           Initialise Software
;*****************************************

                org    0x00 
                bsf    STATUS,RP0
                clrf   PORTA
                movlw  0x06          ;PORTA Bxxx00110
                movwf  TRISA 
 
                clrf   PORTB
                movlw  0x00          ;PORTB B00000000
                movwf  TRISB
                bcf    STATUS,RP0

;*****************************************
;	 Main from parallel PORTB
;*****************************************
  	
Main            btfsc   PORTA,RX         ;Received a start bit ?
                goto    Main             ;No, wait.
                call    RxChar           ;Yes, read the byte on receiving...
                call    ready
                ;call    TxChar          ;TX data
                goto    Main          

;*****************************************
; Delay subroutine
; W = Requested delay time in ms (clock = 10MHz)
;*****************************************

msDelay         movwf   msDelayCounter+1
                clrf    msDelayCounter+0 ; 1 ms (about) internal loop
                
msDelayLoop     decfsz  msDelayCounter+0,F
                nop
                goto    msDelayLoop
                nop
                decfsz  msDelayCounter+1,F
                goto    msDelayLoop

                return

;*****************************************
; Send MIDI DATA
; Input: W = Character to send
;*****************************************

TxChar          movwf   ShiftReg
                movlw   8               ;Data lenght
                movwf   BitCount
                bcf     PORTA,TX        ;Send start bit
                nop
                nop
                nop
                nop
                nop
                nop
                nop
                DELAY   BIT_DELAY       ;Tx loop

TxLoop          btfss   ShiftReg,0
                goto    TxLo
                nop
                bsf     PORTA,TX
                goto    cTx

TxLo            bcf     PORTA,TX
                goto    cTx

cTx             rrf     ShiftReg,F
                nop
                DELAY   BIT_DELAY
                decfsz  BitCount,F
                goto    TxLoop
                nop
                nop
                nop
                nop
                bsf     PORTA,TX                ;Stop bit
                DELAY   BIT_DELAY
                DELAY   2
                nop
                bsf     PORTA,TX
                DELAY   BIT_DELAY
                DELAY   2
                return


;*****************************************
; Receive MIDI DATA
; ShiftReg:  Received character
;*****************************************

RxChar          clrf    ShiftReg
                movlw   8                       ;Data lenght
                movwf   BitCount
                DELAY   BIT_DELAY+BIT_DELAY/2   ;Wait 1.5 bit
                
wDB             btfss   PORTA,RX ;Loop di lettura dei bit dati
                goto    RxBitL

RxBitH          bsf     STATUS,C
                nop
                goto    RxShift

RxBitL          bcf     STATUS,C
                goto    RxShift

RxShift         rrf     ShiftReg,F
                nop
                DELAY   BIT_DELAY
                decfsz  BitCount,F
                goto    wDB
                return

;*****************************************
;	 Send byte parallel on PORTB
;*****************************************

ready           movf    ShiftReg,w       ;buffer ready
                movwf   PORTB  
                return

                END

